// Class  : SimpleServerConnection
// Author : Russ Ethington
// Version: 2/15/96 for Java 1.0

import java.lang.*;
import java.net.*;
import java.io.*;  

public class SimpleServerConnection implements Runnable, Cloneable
{
   private ServerSocket      server = null;
   private Socket            socket = null;
   private Thread            life = null;
   private Thread            twin = null;
   
   private DataInputStream   in = null;
   private PrintStream       out = null;
   
   private boolean           original = true;
   private boolean           runState = false;
   private boolean           connectState = false;
   final int                 defaultPort = 4646;
   
   public static void main(String args[])
   {
      new SimpleServerConnection().start();
      System.out.println("Ready.");
   }

   public SimpleServerConnection () { }

   public void start() { start(defaultPort); }

   public void start(int thePort)
   {
      if (life == null)
      {
         life = new Thread(this);
         life.setPriority(Thread.MAX_PRIORITY);
      }
      try 
         {server = new ServerSocket(thePort);}
      catch (IOException e)
      {
         trouble("Could not create new server socket...");
         return;
      }            
      life.start();
   }

   public void stop()
   {
      if (life != null) 
         life.stop();
      shutdown();
   }

   public boolean isRunning() { return runState; }

   public boolean isConnected() { return connectState; }

   public void run()
   {
      if (original) // this is the primary server thread.
      {
         runState = true;
         while (true)
         {
            try
               {socket = server.accept();}
            catch (IOException e)
            {
               trouble("Could not accept incoming calls...");
               shutdown();
               return;
            }

            SimpleServerConnection copy = null;
            try
               {copy = (SimpleServerConnection) clone();}
            catch (CloneNotSupportedException e)
               {trouble("Unexpected 'CloneNotSupported' exception...");}

            copy.life = null;
            copy.server = null;
            copy.original = false;
            copy.runState = false;
            twin = new Thread(copy);
            twin.start();
         }
      }
      else // this is the twin clone coming to life!
      {
         runState = true;
         connectState = true;
         try
         {
            in = new DataInputStream(
                     new BufferedInputStream(socket.getInputStream()));
            out = new PrintStream(
                      new BufferedOutputStream(socket.getOutputStream()), true);
         }
         catch (IOException e)
         {
            trouble("Could not create data streams...");
            shutdown();
            return;
         }
         handleSession(in, out);
         shutdown();
      }
   }

   public void shutdown()
   {
      life = null;
      twin = null;
      runState = false;
      if (connectState)
      {
         try
         {
            in.close();
            out.close();
         }  catch (IOException e) { }
         connectState = false;
         if (socket != null)
         {
            try {socket.close();}
            catch (Exception e) { }
         }
      }
      socket = null;
      server = null;
      
      in = null;
      out = null;
   }

   public void trouble(String message) { System.out.println(message); }

   public void handleSession(DataInputStream in, PrintStream out)
   {
      System.out.println("SimpleServerConnection:handleSession()");

      // This should be overridden in a subclass. If it gets called,
      // perhaps the subclass was originally based on NetworkServer,
      // so try calling it's (near) equivalent: serviceRequest().

      serviceRequest();
   }

   //
   // These are for semi-compatibility with sun.net.NetworkServer.
   //

   public boolean clientIsOpen() { return connectState; }
   public void startServer(int port) { start(port); }
   public void serviceRequest() { }
   public PrintStream clientOutput() { return out; }
   public InputStream clientInput() { return in; }
   public void close() { shutdown(); }
}
